@@ -34,7 +34,7 @@ if rbconfig.libsecret_enabled:
 	# Till libsecret completely replaces gnome-keyring, we'll fall back to not
 	# saving the password if libsecret can't be found. This code can be removed later.
 	try:
-		from gi.repository import Secret, SecretUnstable
+		from gi.repository import Secret
 		# We need to be able to fetch passwords stored by libgnome-keyring, so we use
 		# a schema with SECRET_SCHEMA_DONT_MATCH_NAME set.
 		# See: http://developer.gnome.org/libsecret/unstable/migrating-schemas.html
@@ -58,17 +58,17 @@ def instance():
 
 class MagnatuneAccount(object):
 	def __init__(self):
-		self.settings = Gio.Settings("org.gnome.rhythmbox.plugins.magnatune")
+		self.settings = Gio.Settings.new("org.gnome.rhythmbox.plugins.magnatune")
 		self.secret = None
 		self.keyring_attributes = {"rhythmbox-plugin": "magnatune"}
 		if Secret is None:
 			print ("Account details will not be saved because libsecret was not found")
 			return
 		# Haha.
-		self.secret_service = SecretUnstable.Service.get_sync(SecretUnstable.ServiceFlags.OPEN_SESSION, None)
+		self.secret_service = Secret.Service.get_sync(Secret.ServiceFlags.OPEN_SESSION, None)
 		items = self.secret_service.search_sync(MAGNATUNE_SCHEMA,
 							self.keyring_attributes,
-							SecretUnstable.SearchFlags.LOAD_SECRETS,
+							Secret.SearchFlags.LOAD_SECRETS,
 							None)
 		if not items:
 			# The Python API doesn't seem to have a way to differentiate between errors and no results.
@@ -88,7 +88,7 @@ class MagnatuneAccount(object):
 			return ('none', None, None)
 
 	def update(self, username, password):
-		secret = '\n'.join((username, password)).encode("utf-8")
+		secret = '\n'.join((username, password))
 		if secret == self.secret:
 			print ("Account details not changed")
 			return
